<?php
/**
* Plugin Name: Giants Dedi Server Status
* Plugin URI: http://www.farming-simulator.com/dlc-detail-fs13-ds.php
* Description: Displays GIANTS Dedicated Server status. Insert [serverlist] on any page to include a server list.
* Author: GIANTS Software
* Author URI: http://www.giants-software.com/
* Version: 1.0
*/
/**
 * Dedicated Server Stats
 *
 * @author  Daniel Evans
 * @date  04/10/2013
 * 
 * Copyright (c) 2008-2013 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2013 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

function serverstats_init() {
    $plugin_dir = dirname( plugin_basename( __FILE__ ) );
    load_plugin_textdomain( 'dedi-server-stats', false, $plugin_dir );
}
add_action('plugins_loaded', 'serverstats_init');

/** Utility Methods */
function loadFileHTTPSocket($domain, $path, $timeout, $port) {
    $fp = fsockopen($domain, $port, $errno, $errstr, $timeout);
    if ($fp) {
        // Make request
        $out = "GET ". $path . " HTTP/1.0\r\n";
        $out .= "Host: ". $domain . "\r\n";
        $out .= "Connection: Close\r\n\r\n";
        fwrite($fp, $out);
    
        // Get response
        $resp = "";
        while (!feof($fp)) {
            $resp .= fgets($fp, 256);
        }
        fclose($fp);
                    
        // Check status is 200
        if(preg_match("/HTTP\/1\.\d\s(\d+)/", $resp, $matches) && $matches[1] == 200) {   
            // Load xml as object
            $parts = explode("\r\n\r\n", $resp);
            $temp = "";
            for ($i=1;$i<count($parts);$i++) {
                $temp .= $parts[$i];
            }
            return $temp;
        }
    }
    return false;
}

function getServerStatsSimpleXML($id, $server) {
    $url = strtolower($server);
    if(substr($url, 0, 7) == "http://") {
        $url = substr($url, 7);
    }
    $data = explode('/', $url, 2);
    $data_host = explode(":", $data[0]);
    $port = 80;
    $host = $data_host[0];
    if(count($data_host) > 1) {
        $port = $data_host[1];
    }
    $stats_url = "/{$data[1]}";
    $cacheFile = "dedicated-server-stats-{$id}-{$host}-{$port}.cached";
    $cacheTimeout = 60*2;
    if(file_exists($cacheFile) && filemtime($cacheFile) > (time() - ($cacheTimeout + rand(0, 10)))) {
        $xmlStr = file_get_contents($cacheFile);
    } else {
        $xmlStr = loadFileHTTPSocket($host, $stats_url . $code, 4, $port);
        
        if ($xmlStr) {
            $fp = fopen($cacheFile, "w");
            fwrite($fp, $xmlStr);
            fclose($fp);            
        }
    }
    return simplexml_load_string($xmlStr);
}

class DediServerStatusWidget extends WP_Widget {
    function __construct() {
        parent::__construct(
            'server_status_widget',
            __('GIANTS Dedi Server Status', 'dedi-server-stats'),
            array( 'description' => __('Widget that displays Dedi Server Status', 'dedi-server-stats'))
        );
    }
    
    public function widget($args, $instance) {
        $title = $instance['title'];
        $server = $instance['server_address'];
        $list_mods = $instance['list_mods'];
        $show_details = $instance['show_details'];
        $list_players = $instance['list_players'];
        $in_server_list = 'server-list-sidebar' == $args['id'];
        echo $args['before_widget'];
        if ( ! empty( $title ) )
            echo $args['before_title'] . $title . $args['after_title'];
        
        if (! empty( $server )) {
            $data = getServerStatsSimpleXML( $args['widget_id'], $server );
            if ($data && (! empty($data->Slots))) {
                echo '<strong>'.__('Name', 'dedi-server-stats').':</strong> ' . $data['name'] . '<br/>';
                echo '<strong>'.__('Game', 'dedi-server-stats').':</strong> ' . $data['game'] . '<br/>';
                $usedSlots = 0; $totalSlots = count($data->Slots->Player);
                foreach ($data->Slots->Player as $player) {
                    if ( $player['isUsed'] != "true" ) continue;
                    $usedSlots++;
                }
                echo '<strong>'.__('Players', 'dedi-server-stats').':</strong> ' . "{$usedSlots}/{$totalSlots}" . '<br/>';
                echo '<strong>'.__('Map', 'dedi-server-stats').':</strong> ' . $data['mapName'] . '<br/>';
                echo '<strong>'.__('Money', 'dedi-server-stats').':</strong> ' . $data['money'] . ' ' .__('$', 'dedi-server-stats'). '<br/>';
                echo '<strong>'.__('Server', 'dedi-server-stats').':</strong> ' . $data['server'] . '<br/>';
                if ($show_details) {
                    if ($list_players && $usedSlots > 0) {
                        echo '<table' . ($in_server_list ? '' : ' style="width: 100%"') . '><thead>';
                        echo '<th>'.__('Player', 'dedi-server-stats').'</th>';
                        echo '<th>'.__('Time', 'dedi-server-stats').'</th>';
                        if($in_server_list) {
                            echo '<th>'.__('Admin', 'dedi-server-stats').'</th>';
                        }
                        echo '</thead><tbody>';
                        foreach ($data->Slots->Player as $player) {
                            if ( $player['isUsed'] != "true" ) continue; ?>
                            <tr>
                                <td><?php echo $player; ?></td>
                                <?php $hours = floor($player["uptime"]/60); $minutes = floor($player["uptime"]-($hours*60)); ?>
                                <td><?php echo "{$hours}h {$minutes}m"; ?></td>
                                <?php if($in_server_list) { ?>
                                <td><?php echo (($player["isAdmin"] == "true")?__('Yes', 'dedi-server-stats'):__('No', 'dedi-server-stats')); ?></td>
                                <?php } ?>
                            </tr>
                        <?php }
                        echo '</table>';
                    }
                    if ($list_mods && $in_server_list && count($data->Mods->Mod) > 0) {
                        echo '<table><thead>';
                        echo '<th>'.__('Mod', 'dedi-server-stats').'</th>';
                        echo '<th>'.__('Author (Version)', 'dedi-server-stats').'</th>';
                        echo '</thead><tbody>';
                        foreach ($data->Mods->Mod as $mod) { ?>
                            <tr>
                                <td><?php echo $mod; ?></td>
                                <td><?php echo $mod["author"]; ?> (v<?php echo $mod["version"]; ?>)</td>
                            </tr>
                        <?php }
                        echo '</table>';
                    } 
                }
                else {
                    if ($list_players) {
                        echo '<strong>'.__('Players', 'dedi-server-stats').':</strong>';
                        if($usedSlots > 0) {
                            echo '<ul>';
                            foreach ($data->Slots->Player as $player) {
                                if ( $player['isUsed'] != "true" ) continue;
                                echo "<li>{$player}</li>";
                            }
                            echo '</ul>';
                        }
                    }
                    if ($list_mods && $in_server_list) {
                        echo '<strong>'.__('Mods', 'dedi-server-stats').'</strong>';
                        if(count($data->Mods->mod) > 0) {
                            echo '<ul>';
                            foreach ($data->Mods->Mod as $mod) {
                                echo "<li>{$mod}</li>";
                            }
                            echo '</ul>';
                        }
                    } 
                }   
                echo '<p>Powered by <a href="http://www.giants-software.com">GIANTS Software</a>.</p>';
            }
            else {
                echo '<p>'.__('Server Unavailable').'.</p>';
            }
        }
        
        echo $args['after_widget'];
    }
    
    public function form( $instance ) {
        if ( isset( $instance[ 'server_address' ] ) ) {
            $address = $instance[ 'server_address' ];
        }
        else { $address = ''; }
        if ( isset( $instance[ 'title' ] ) ) {
            $title = $instance[ 'title' ];
        }
        else { $title = ''; }
        if ( isset( $instance[ 'list_mods' ] ) ) {
            $list_mods = $instance[ 'list_mods' ];
        }
        else { $list_mods = FALSE; }
        if ( isset( $instance[ 'list_players' ] ) ) {
            $list_players = $instance[ 'list_players' ];
        }
        else { $list_players = FALSE; }
        if ( isset( $instance[ 'show_details' ] ) ) {
            $show_details = $instance[ 'show_details' ];
        }
        else { $show_details = FALSE; }
        ?>
        <p>
        <label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title', 'dedi-server-stats' ); ?>:</label> 
        <input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" />
        </p>
        <p>
        <label for="<?php echo $this->get_field_id( 'server_address' ); ?>"><?php _e( 'Server Stats URL feed', 'dedi-server-stats' ); ?>:</label> 
        <input class="widefat" id="<?php echo $this->get_field_id( 'server_address' ); ?>" name="<?php echo $this->get_field_name( 'server_address' ); ?>" type="text" value="<?php echo esc_attr( $address ); ?>" />
        </p>
        <p>
        <label for="<?php echo $this->get_field_id( 'list_players' ); ?>"><?php _e( 'List Players', 'dedi-server-stats' ); ?>:</label> 
        <input class="widefat" id="<?php echo $this->get_field_id( 'list_players' ); ?>" name="<?php echo $this->get_field_name( 'list_players' ); ?>" type="checkbox" <?php checked($list_players); ?> />
        </p>
        <p>
        <label for="<?php echo $this->get_field_id( 'list_mods' ); ?>"><?php _e( 'List Mods', 'dedi-server-stats' ); ?>:</label> 
        <input class="widefat" id="<?php echo $this->get_field_id( 'list_mods' ); ?>" name="<?php echo $this->get_field_name( 'list_mods' ); ?>" type="checkbox" <?php checked($list_mods); ?> />
        </p>
        <p>
        <label for="<?php echo $this->get_field_id( 'show_details' ); ?>"><?php _e( 'Show Details', 'dedi-server-stats' ); ?>:</label> 
        <input class="widefat" id="<?php echo $this->get_field_id( 'show_details' ); ?>" name="<?php echo $this->get_field_name( 'show_details' ); ?>" type="checkbox" <?php checked($show_details); ?> />
        </p>
        <?php 
    }

    public function update( $new_instance, $old_instance ) {
        $instance = array();
        $instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
        $instance['server_address'] = ( ! empty( $new_instance['server_address'] ) ) ? strip_tags( $new_instance['server_address'] ) : '';
        $instance['list_mods'] = ( ! empty( $new_instance['list_mods'] ) ) ? $new_instance['list_mods'] == TRUE : FALSE;
        $instance['list_players'] = ( ! empty( $new_instance['list_players'] ) ) ? $new_instance['list_players'] == TRUE : FALSE;
        $instance['show_details'] = ( ! empty( $new_instance['show_details'] ) ) ? $new_instance['show_details'] == TRUE : FALSE;

        return $instance;
    }
};

function register_server_status_widget() {
    register_widget( 'DediServerStatusWidget' );
}
add_action( 'widgets_init', 'register_server_status_widget' );

// Register shortcode for server list
function serverlist_func($atts) {
    $widgets = get_option('widget_server_status_widget');
    foreach($widgets as $id=>$w) {
        the_widget('DediServerStatusWidget', $w, array('id' => 'server-list-sidebar', 'widget_id' => $id));
    }
}
add_shortcode('serverlist', 'serverlist_func');